# -*- coding: utf-8 -*-

__title__ = 'nanodlna'
__version__ = '0.2.0'
__short_version__ = '.'.join(__version__.split('.')[:2])
__author__ = 'Gabriel Magno'
__license__ = 'MIT'
__copyright__ = 'Copyright 2016, Gabriel Magno'
